METRIC_MAPPING = {
  "Gross Revenue": {
    "Revenues, incl Govt Payments": {
      "Crop Insurance Payments": {},
      "Government Payments": {
        "Government Payments, Other": [],
        "USDA Agricultural Risk and Price Loss Coverage Program Payments": []
      },
      "Production Revenue": {
        "Producer Prices": [],
        "Yield": []
      },
      "Production Revenue, Projected": {
        "Producer Prices, Projected": [],
        "Yield Assumption": [],
        "Yield, Historical Average": []
      },
      "Revenues, Other": {}
    },
    "Total Revenues, incl Govt Payments": {}
  },
  "Total Costs": {
    "Expenses": {
      "Expenses, excl Land": {},
      "Expenses, Other": {},
      "Operating Expenses": {
        "Variable Labor Expenses": [],
        "Bag/Tie Expenses": [],
        "Boll Weevil Assessment Expenses": [],
        "Boll Weevil Eradication Expenses": [],
        "Boll Weevil Program Expenses": [],
        "Boll Worm Eradication Expenses": [],
        "Chemical Application": [],
        "Chemical Application Cost": [],
        "Chemical Expenses": [
          "Additive Expenses",
          "2-4D Amine Chemical Expenses",
          "Chemical Expenses, Acid",
          "Chemical Expenses, Prep",
          "Fungicide Expenses",
          "Herbicide Expenses",
          "Inoculant Expenses",
          "Insecticide Expenses",
          "Pesticide Expenses",
          "Surfactant Expenses"
        ],
        "Crop Consulting Expenses": [],
        "Crop Growing Expenses": [],
        "Crop Insurance Premium Expenses": [],
        "Custom Work Expenses": [],
        "Drying Expenses": [],
        "Fertilizer Application": [],
        "Fertilizer Application Cost": [],
        "Fertilizer Application Expenses": [
          "Ammonia Fertilizer Application Expenses",
          "Anhydrous Ammonia Fertilizer Application Expenses",
          "Dry Fertilizer Application Expenses",
          "Fertilizer Application Expenses, Broadcast Blend",
          "Fertilizer Application Expenses, Broadcast P & K",
          "Lime Fertilizer Application Expenses",
          "Liquid Fertilizer Application Expenses, High",
          "Liquid Fertilizer Application Expenses, Low",
          "Liquid Fertilizer Rig Expenses"
        ],
        "Fertilizer Expenses": [
          "0-60-0 Fertilizer Expenses",
          "2-4D Amine Fertilizer Expenses",
          "24-6-2 Fertilizer Expenses",
          "24-8-0 Fertilizer Expenses",
          "32-11-3 6S Fertilizer Expenses",
          "Ammonia Fertilizer Expenses",
          "Ammonium Phosphate Fertilizer Expenses",
          "Ammonium Polyphosphate Fertilizer Expenses",
          "Anhydrous Ammonia Fertilizer Expenses",
          "Complete NPK (18-18-18) Fertilizer Expenses",
          "Custom Fertilizer Expenses",
          "Diammonium Phosphate (DAP) Fertilizer Expenses",
          "Dry Nitrogen Fertilizer Expenses",
          "Dry Phosphorous Fertilizer Expenses",
          "Fertilizer, Lime, and Soil Conditioner Expenses",
          "Lime Fertilizer Expenses",
          "Liquid Nitrogen Fertilzer Expenses",
          "Liquid Phosphorous Fertilizer Expenses",
          "Manure Fertilizer Expenses",
          "Monoammonium Phosphate (MAP) Fertilizer Expenses",
          "Monoammonium Phosphate Fertilizer Expenses",
          "Nitrogen Fertilizer Expenses",
          "NPK Fertilizer Expenses",
          "NPZnS Fertilizer Expenses",
          "Phosphate Fertilizer Expenses",
          "Phosphoric Acid Fertilizer Expenses",
          "Phosphorous Fertilizer Expenses",
          "Potash Fertilizer Expenses",
          "Potassium Fertilizer Expenses",
          "Rod Weeder and Fertilizer Expenses",
          "Rod Weeder Expenses",
          "Sulfur Fertilizer Expenses",
          "Triple Super Phosphate Fertilizer Expenses",
          "Urea Ammonium Nitrate Fertilizer Expenses",
          "Urea Ammonium Nitrate Fertilizer Expenses, Applied at or before the Flowering Stage",
          "Urea Ammonium Nitrate Fertilizer Expenses, Applied by Pivot",
          "Urea Fertilizer Expenses"
        ],
        "Field Operations Expenses": [
          "Caparol Pesticide Expenses",
          "Cart Expenses",
          "Chop Stalks Expenses",
          "Combine Expenses",
          "Combine Harvest Soybeans",
          "Combine Irrigated Corn Harvest Expenses",
          "Combine Small Grain Harvest Expenses",
          "Combine Soybean Harvest Expenses",
          "Dessicant/Defoliant Expenses",
          "Disk Expenses",
          "Drill Expenses",
          "Field Cultivation Expenses",
          "Field Operations Expenses, One-Pass Tillage",
          "Grain Harvest Field Operations Expenses",
          "Grain Harvest Operations Expenses, Combine",
          "Irrigation Expenses",
          "No-Till Drilling Expenses",
          "Pipe Irrigation Expenses",
          "Pre-Harvest Field Operations Expenses",
          "Ridge Cultivation Expenses",
          "Ridge Cultivation/Ditch Expenses",
          "Row Crop Cultivation Expenses",
          "Spray Expenses",
          "Spray Fertilizer and Herbicide Application Expenses",
          "Spray Fertilizer Application Expenses",
          "Spray Fertilizer Expenses",
          "Spray Herbicide Application Expenses",
          "Spray Spring Burndown Herbicide Expenses",
          "Spread Fertilizer Application Expenses",
          "Straw Harvest Field Operations Expenses",
          "Sweep Plowing Expenses"
        ],
        "Fuel and Machinery Expenses": [],
        "Fuel Expenses": [
          "Diesel Fuel Expenses, General Use Equipment",
          "Diesel Fuel Expenses, Tractors",
          "Fuel Expenses, General Use Equipment",
          "Fuel Expenses, incl Electricity",
          "Fuel Expenses, incl. Electricity",
          "Fuel Expenses, Tractors",
          "Vehicle Expenses"
        ],
        "Fungicide Application Expenses": [],
        "Ginning and Hauling Expenses": [],
        "Ginning Expenses": [
          "Custom Ginning Expenses"
        ],
        "GinStar Defoliation Expenses": [],
        "Harvest Aid Application Expenses": [
          "Harvest Aid Application Expenses, Drip",
          "Harvest Aid Expenses, Defoliant",
          "Harvest Aid Expenses, Prep"
        ],
        "Harvest and Hauling Expenses": [],
        "Harvest Expenses": [],
        "Harvest Field Operations Expenses": [],
        "Hauling Expenses": [],
        "Herbicide and Application Expenses": [],
        "Herbicide Application Expenses": [
          "Herbicide Application Expenses, Spray",
          "Herbicide Application Expenses, Aerial Spray",
          "Herbicide Expenses, First Application",
          "Herbicide Expenses, Second Application",
          "Post-Plant Herbicide Application Expenses",
          "Pre-Plant Herbicide Application Expenses",
          "Spray Burndown Herbicide Application Expenses"
        ],
        "Insecticide Application Expenses": [
          "Apply by Air Insecticide Expenses",
          "Insecticide Expenses, First Application",
          "Insecticide Expenses, Second Application"
        ],
        "Insecticide and Application Expenses": [],
        "Insecticide Expenses": [
          "Brigade 2EC Insecticide Expenses",
          "Brigade Insecticide Expenses",
          "Capture LFR Insecticide Expenses",
          "Centric Insecticide Expenses",
          "Cutworm Control Insecticide Expenses",
          "Entrust Insecticide Expenses",
          "Flea Hopper Control Insecticide Expenses",
          "Greenbug Insecticide Expenses",
          "Insurance Expenses, Miscellaneous",
          "Intruder Insecticide Expenses",
          "Karate Z Insecticide Expenses",
          "Lannate LV Insecticide Expenses",
          "Lorsban 4E Insecticide Expenses",
          "Lorsban Advanced Insecticide Expenses",
          "Macho 4F Insecticide Expenses",
          "Miticide Expenses",
          "Mustang Maxx Insecticide Expenses",
          "Oberon 2 SC Insecticide Expenses",
          "Oberon 4 SC Insecticide Expenses",
          "Pilot Insecticide Expenses",
          "Planting Insecticide Expenses",
          "Pre-Plant Insecticide Expenses",
          "Prevathon Insecticide Expenses",
          "Sivanto Insecticide Expenses",
          "Spider Mite Insecticide Expenses",
          "Stinkbug Control Insecticide Expenses",
          "TBW-BAW-AW-BW Insecticide Expenses",
          "Tracer Insecticide Expenses",
          "Transform Insecticide Expenses",
          "Vydate Insecticide Expenses",
          "Warrior II/Zeon Insecticide Expenses",
          "Worm Control Insecticide Expenses"
        ],
        "Irrigation Expenses": [
          "Ditch Irrigation Expenses",
          "Irrigation Energy Expenses",
          "Irrigation Expenses, Energy Cost",
          "Irrigation Expenses, Labor",
          "Irrigation Expenses, Water Cost",
          "Irrigation Labor Expenses",
          "Pivot Irrigation Expenses"
        ],
        "Labor Expenses": [
          "Custom Hand Hoeing Expenses",
          "Labor Expenses, Hand Labor",
          "Labor Expenses, Hoeing",
          "Labor Expenses, Salaried",
          "Labor Expenses, Unallocated"
        ],
        "Machine Hire and Custom Work Expenses": [],
        "Machinery and Energy Expenses": [],
        "Marketing Expenses": [],
        "Operating Expenses, Other": [],
        "Operating Interest Expenses": [],
        "Overage Expenses": [],
        "Pick and Module Expenses": [],
        "Picking Expenses": [],
        "Pickup Mileage Charge Expenses": [],
        "Planting Expenses": [
          "No-Till Planting Expenses"
        ],
        "Scouting Expenses": [
          "Scouting Expenses, Insects",
          "Scouting/Consulting Expenses"
        ],
        "Seed Expenses": [
          "Bt and ECB Seed Expenses",
          "Bt, ECB, and RIB Seed Expenses",
          "Bt, ECB, and RW Seed Expenses",
          "Bt, ECB, RR2, and RIB Seed Expenses",
          "Bt, ECB, RR2, LL, and RIB Seed Expenses",
          "Bt, ECB, RW, and RIB Seed Expenses",
          "Bt, ECB, RW, and RR2 Seed Expenses",
          "Bt, ECB, RW, RR2, and RIB Seed Expenses",
          "Bt, ECB, RW, RR2, LL, and RIB Seed Expenses",
          "Certified and Treated Seed Expenses",
          "ECB and RR2 Seed Expenses",
          "Enlist E3 Treated Seed Expenses",
          "Enlist, Bt, ECB, and RIB Seed Expenses",
          "Inoculant Seed Expenses",
          "RR Seed Expenses",
          "RR2 Seed Expenses",
          "RR2 Xtend Seed Expenses",
          "Rye Seed Expenses",
          "Seed Expenses, B2 Auxin Technology",
          "Seed Expenses, B2RF",
          "Seed Expenses, B2RF Technology Fee",
          "Seed Expenses, B2XF",
          "Seed Expenses, BIIRF",
          "Seed Expenses, BIIRRF",
          "Seed Expenses, Bt",
          "Seed Expenses, Bt Technology Fee",
          "Seed Expenses, BtRR",
          "Seed Expenses, Cotton",
          "Seed Expenses, Cowpea",
          "Seed Expenses, GMO",
          "Seed Expenses, Herbicide Resistant",
          "Seed Expenses, Herbicide Technology Fee",
          "Seed Expenses, Herbicide Tolerant",
          "Seed Expenses, Herbicide Tolerant Technology Fee",
          "Seed Expenses, Inoculant",
          "Seed Expenses, Insect Treatment",
          "Seed Expenses, Roundup Ready",
          "Seed Expenses, Stacked Gene",
          "Seed Expenses, Technology Fee",
          "Seed Expenses, Wheat",
          "SmartStax RIB Complete Seed Expenses",
          "Soybeans Seed Expenses",
          "Wheat Seed Expenses"
        ],
        "Soil Test Expenses": [],
        "Storage and Marketing Expenses": [],
        "Storage Expenses": [],
        "Strip and Module Expenses": [],
        "Strip Expenses": [],
        "Transportation Expenses": [],
        "Utility Expenses": [],
        "Variable Expenses, Other": [],
        "Variable Machine Hire Expenses": [],
        "Variable Utility Expenses": []
      },
      "Overhead Expenses": {
        "Building Maintenance Expenses": [],
        "Capital Opportunity Cost": [],
        "Depreciation Expenses": [
          "Building Depreciation Expenses",
          "Capital Investment and Depreciation Expenses",
          "Machinery Depreciation Expenses"
        ],
        "Electricity Expenses": [],
        "Establishment Cost, Equipment": [],
        "Fixed Labor Expenses": [],
        "Fixed Machine Hire Expenses": [],
        "Fixed Utility Expenses": [],
        "General Management Expenses": [],
        "Insurance Premium Expenses": [
          "Building Insurance Premium Expenses",
          "Crop Insurance Premium Expenses, Hail",
          "Crop Insurance Premium Expenses, Herbicide Drift",
          "Miscellaneous Insurance Premium Expenses, SCO and STAX",
          "Multiple Peril Crop Insurance (MPCI) Premium Expenses",
          "Revenue Protection Crop Insurance Premium Expenses, 65%",
          "Revenue Protection Crop Insurance Premium Expenses, 65% YA",
          "Revenue Protection Crop Insurance Premium Expenses, 70%",
          "Revenue Protection Crop Insurance Premium Expenses, 70% SE",
          "Revenue Protection Crop Insurance Premium Expenses, 70% YA",
          "Revenue Protection Crop Insurance Premium Expenses, 70% YE",
          "STAX Crop Insurance Premium Expenses",
          "Supplemental Coverage Option (SCO) Insurance Premium Expenses"
        ],
        "Interest Expenses": [],
        "Land Opportunity Cost": [],
        "Land rent": [],
        "Land Rent and Lease Payment Expenses": [],
        "Machinery and Equipment Overhead Cost, General Use Equipment": [],
        "Machinery and Equipment Overhead Cost, Implements": [],
        "Machinery and Equipment Overhead Cost, Irrigation Equipment": [],
        "Machinery and Equipment Overhead Cost, Tractors": [],
        "Machinery Expenses": [
          "Machine Hire Expenses",
          "Machine Repair Expenses",
          "Machinery and Equipment Investment Expense, General Use Equipment",
          "Machinery and Equipment Investment Expense, Implements",
          "Machinery and Equipment Investment Expense, Irrigation Equipment",
          "Machinery and Equipment Investment Expense, Tractors",
          "Machinery Expenses, Other Labor",
          "Machinery Expenses, Tractors Labor"
        ],
        "Overhead Expenses, Other": [],
        "Professional Fees and Dues": [],
        "Real Estate Taxes": [],
        "Rent and Lease Payment Expenses": [],
        "Repairs and Maintenance Expenses": [],
        "Seed, Chemicals, Insurance, and Miscellaneous Expenses": [],
        "Soil Testing Expenses": [],
        "Taxes and Insurance Expenses": [],
        "Unpaid Labor Opportunity Cost": []
      }
    },
    "Expenses (currency/mass)": {
      "Operating Expenses (currency/mass)": {},
      "Overhead Expenses (currency/mass)": {}
    }
  },
  "Net Income": {
    "Net Profit": {},
    "Net Profit, excl Labor and Management Expenses": {},
    "Net Profit, excl Land Expenses": {},
    "Net Profit, excl Land, Labor and Management Expenses": {},
    "Net Profit, excl Overhead Costs": {},
    "Net Profit, Projected": {},
    "Operating Profit": {}
  }
}



import io
import json
import requests 

import pandas as pd

from grocropclient.constants import ItemNameCategoryList
from grocropclient.crop_budgets_data import METRIC_MAPPING


API_HOST = "api.gro-intelligence.com"
PRECISION = 2

# To get all crop budget tables from the CSV above.
def get_all_crop_budgets(api_host, api_token, crop='', state=''):
  try:
    crop = crop.lower()
    state = state.lower()
  except AttributeError as e:
    raise TypeError(f'crop and state parameters must be strings') from e
  if state:
      state = state.replace(' ', '_') # the source name uses underscore for multiword US states.
  headers = { 'Authorization' : f'Bearer {api_token}' }
  endpoint = 'crop-budget-available-series'
  url = "/".join(["https:", "", api_host, endpoint])
  resp = requests.get(url=url,
                    headers=headers)
  json_resp = json.dumps(resp.json()['data'])
  df = pd.read_json(json_resp, orient='records')
  for col in df.columns:
    camel_case_col = col
    snake_case_col = __camel_case_to_snake_case(col)
    df.rename(columns = {camel_case_col: snake_case_col}, inplace=True)
  # Filter out crop budgets for item_names that are not for Corn or Soybeans crops
  df = df[df['item_name'].apply(lambda x: x.lower()).str.contains(f'{ItemNameCategoryList.CORN.value}|{ItemNameCategoryList.SOYBEANS.value}')]
  return df.loc[df.item_name.apply(lambda x: x.split(',')[0]).str.lower().str.contains(crop) & df.source_name.str.contains(state, case=False)]


def __crop_budget_json_to_df2(crop_budget_as_json: dict) -> pd.DataFrame:
    flattened_metrics = __flatten_to_list(METRIC_MAPPING)
    csv_str = __json_to_csv(crop_budget_as_json, flattened_metrics)
    csv_string_io = io.StringIO(csv_str)
    return pd.read_csv(csv_string_io, sep=',').round(decimals=PRECISION)


# To get a single crop budget, converted into a Pandas dataframe.
def get_crop_budget_as_df(api_host, api_token, source_name, item_name, region_name, productivity):
  headers = { 'Authorization' : f'Bearer {api_token}' }
  params = {'sourceName': source_name, 'itemName': item_name, 'regionName': region_name, 'productivity': productivity}
  endpoint = 'crop-budget'
  url = "/".join(["https:", "", api_host, endpoint])
  resp = requests.get(url=url,
                    headers=headers,
                    params=params)
  return __crop_budget_json_to_df2(resp.json())

def __flatten_to_list(obj, parent = None, res = None):
    if not res:
        res = []
    for key in obj:
        res.append(key)
        if parent:
            prop_name = parent + '.' + key
        else:
            prop_name = key
        
        if isinstance(obj[key], list):
            for item in obj[key]:
                res.append(item)
        elif isinstance(obj[key], dict):
            __flatten_to_list(obj[key], prop_name, res)

    return res


def __json_to_csv(json_input: dict, row_order):
    csv_rows = []
    title = '"Crop Budget"'
    
    first_key = list(json_input.keys())[0]
    first_row = json_input[first_key]['data']
    keys = list(first_row.keys())
    header = [title] + keys
    header_str = ','.join(header)
    csv_rows.append(header_str)
    
    for name in row_order:
        if name in json_input:
            row_name = [f'"{name}"']
            row = json_input[name]['data']
            values = list(row.values())
            values_str = [str(value) for value in values]
            row_data = row_name + values_str
            row_str = ','.join(row_data)
            csv_rows.append(row_str)
    return '\n'.join(csv_rows)


def __camel_case_to_snake_case(col):
    res = [col[0].lower()]
    for c in col[1:]:
        if c in ('ABCDEFGHIJKLMNOPQRSTUVWXYZ'):
            res.append('_')
            res.append(c.lower())
        else:
            res.append(c)
     
    return ''.join(res)
